-- FailBotSpam - an addon to suppress annoying Recount damage output stuff and 
-- instead condense it into clickable chat links
--
-- Written by Wrug and Cybey
--
-- Code inspiration from the WelcomeHome tutorial, Deadly Boss Mods and GreedBeacon

FailBotSpam = LibStub("AceAddon-3.0"):NewAddon("FailBotSpam", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("FailBotSpam")

local defaults = {
    profile =  {
	failBotAction = 2,
	debug = false,
	captureDelay = 0.5,
	highlight = true,
    },
}

local failBotSpamLines = {
	"^(%a+) fails at (.*) (%()(%d+)(%))$",
}

local fails = {}
local timers = {}

local options = {
	name = "FailBotSpam",
	handler = FailBotSpam,
	type = "group",
	args = {
--		debug = {
--			type = "toggle",
--			name = L["Enable Debug"],
--			desc = L["Enable the debug function"],
--			set = function(info, value)
--				FailBotSpam.db.profile.debug = value
--			end,
--			get = function(info)
--				return FailBotSpam.db.profile.debug
--			end
--		},
		highlight = {
			type = "toggle",
			name = L["Highlight myself"],
			desc = L["Enable the highlighting of your own name"],
			set = function(info, value)
				FailBotSpam.db.profile.highlight = value
			end,
			get = function(info)
				return FailBotSpam.db.profile.highlight
			end
		},
		failBotAction = {
			type = "select",
			name = L["Select action"],
			desc = L["Selects the action to perform when encountering FailBot lines in chat"],
			set = function(info, value)
				FailBotSpam.db.profile.failBotAction = value
			end,
			get = function(info)
				return FailBotSpam.db.profile.failBotAction
			end,
			values = {	
						[1] = L["Do Nothing"],
						[2] = L["Compress"],
						[3] = L["Suppress"]
					}
		},
		captureDelay = {
			type = "range",
			name = L["Capture Delay"],
			desc = L["How many seconds the addon waits after the first FailBot line before it assumes spam burst is over. 1 seems to work in most cases"],
			set = function(info, value)
				FailBotSpam.db.profile.captureDelay = value
			end,
			get = function(info)
				return FailBotSpam.db.profile.captureDelay
			end,
			min = 0.5,
			max = 2,
			step = 0.1
		},
	},
}

function FailBotSpam:OnInitialize()
	-- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("FailBotSpamDB", defaults, "Default")

	LibStub("AceConfig-3.0"):RegisterOptionsTable("FailBotSpam", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FailBotSpam", "FailBotSpam")
	self:RegisterChatCommand("fbs", "ChatCommand")
	self:RegisterChatCommand("failbotspam", "ChatCommand")
end

function FailBotSpam:OnEnable()
	-- Called when the addon is enabled
	ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID", self.ParseChatEvent)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER", self.ParseChatEvent)
end

function FailBotSpam:OnDisable()
	-- Called when the addon is disabled
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID", self.ParseChatEvent)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_RAID_LEADER", self.ParseChatEvent)
end

function FailBotSpam:Debug(...)
	-- Print the message to screen if debug is turned on
	if FailBotSpam.db.profile.debug then
		self:Print(...)
	end
end

function FailBotSpam:ChatCommand(input)
	-- Open dialog if no parameters, else do something with the parameters
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(FailBotSpam, "fbs", "FailBotSpam", input)
	end
end

function FailBotSpam:filterLine(event, source, msg, ...)
	local isSpam = false
	local curTime = GetTime()

	for k,v in ipairs(failBotSpamLines) do
		if msg:match(v) then
			FailBotSpam:Debug(msg,"matched",v)
			local k, failFound
			
			local firstPos, endPos, character, ability, bracket, number = string.find(msg, v)
			
			for i,j in ipairs(fails) do
				local elapsed = curTime - j.time
				if j.src == source and j.abi == ability and elapsed < FailBotSpam.db.profile.captureDelay then
					-- found the fail, save it
					k = j
				end
			end

			local toInsert = true
			
			if character == UnitName("player") and FailBotSpam.db.profile.highlight then
				character = ">|cffff0000"..character.."|r<"
			end
			
			local dataInsert = {char = character, num = number}
			if not k then
				-- No fail found yet, create it!
				local newFail = {src = source, abi = ability, time = curTime, data = {dataInsert}}
				table.insert(fails, newFail)
				toInsert = false
				self:ScheduleTimer("checkFailBotBurst", (FailBotSpam.db.profile.captureDelay + 0.1))
			else
				-- Fail found, check if the data is already in there
				for a,b in ipairs(k.data) do
					if (b.char == character) then
						toInsert = false
					end
				end
			end
					
			if toInsert then
				table.insert(k.data, dataInsert)
			end
			
			return true
		end
	end

	return false
end

function FailBotSpam:checkFailBotBurst()
	local newMsg = ""
	for i,j in ipairs(fails) do
		local curTime = GetTime()
		local elapsed = curTime - j.time
		if elapsed >= FailBotSpam.db.profile.captureDelay then
			newMsg = "Failed at "..j.abi..": "
			for a,b in ipairs(j.data) do
				newMsg = newMsg..b.char.." ("..b.num.."), "
			end
			newMsg = string.sub(newMsg, 0, (string.len(newMsg) - 2))

			table.remove(fails, i)
			
			if (FailBotSpam.db.profile.failBotAction == 2) then
				self:Print(newMsg)
			end
		end
	end
	
	return
end

function FailBotSpam:ParseChatEvent(event, msg, sender, ...)
	-- if not isSpam and (> doNothing) then
	if FailBotSpam.db.profile.failBotAction > 1 then
		if FailBotSpam:filterLine(event,sender,msg) then
			return true, msg, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11
		end
	end

	return false
end
